@@ -8,7 +8,8 @@
 
 from fastavro.six import MemoryIO
 from struct import error as StructError
-from zlib import decompress
+import bz2
+import zlib
 import datetime
 from decimal import localcontext, Decimal
 from uuid import UUID
@@ -23,7 +24,7 @@ from .six import (
 from .schema import extract_record_type, extract_logical_type, parse_schema
 from ._schema_common import SCHEMA_DEFS
 from ._read_common import (
-    SchemaResolutionError, MAGIC, SYNC_SIZE, HEADER_SCHEMA
+    SchemaResolutionError, MAGIC, SYNC_SIZE, HEADER_SCHEMA, missing_codec_lib
 )
 from ._timezone import utc
 from .const import (
@@ -462,12 +463,19 @@ def deflate_read_block(decoder):
     data = decoder.read_bytes()
     # -15 is the log of the window size; negative indicates "raw" (no
     # zlib headers) decompression.  See zlib.h.
-    return MemoryIO(decompress(data, -15))
+    return MemoryIO(zlib.decompress(data, -15))
+
+
+def bzip2_read_block(decoder):
+    """Read block in "bzip2" codec."""
+    data = decoder.read_bytes()
+    return MemoryIO(bz2.decompress(data))
 
 
 BLOCK_READERS = {
     'null': null_read_block,
-    'deflate': deflate_read_block
+    'deflate': deflate_read_block,
+    'bzip2': bzip2_read_block,
 }
 
 
@@ -480,10 +488,24 @@ def snappy_read_block(decoder):
 
 try:
     import snappy
-
-    BLOCK_READERS['snappy'] = snappy_read_block
 except ImportError:
-    pass
+    BLOCK_READERS['snappy'] = missing_codec_lib("snappy", "python-snappy")
+else:
+    BLOCK_READERS['snappy'] = snappy_read_block
+
+
+def zstandard_read_block(decoder):
+    length = read_long(decoder)
+    data = decoder.read_fixed(length)
+    return MemoryIO(zstd.ZstdDecompressor().decompress(data))
+
+
+try:
+    import zstandard as zstd
+except ImportError:
+    BLOCK_READERS["zstandard"] = missing_codec_lib("zstandard", "zstandard")
+else:
+    BLOCK_READERS["zstandard"] = zstandard_read_block
 
 
 def _iter_avro_records(decoder, header, codec, writer_schema, reader_schema):
